#include <stdio.h>
#include <stdlib.h>
#include "variables.h"

//Funcion Calc_N_Vertices: Calcula la cantidad de vrtices que tiene el grafo.
//Variables de entrada   : Descriptor del archivo con el grafo.
//Valor devuelto	 : Cantidad de vrtices en el grafo.
int Calc_N_Vertices (FILE *GRAFODATA)
{
  int maximo = -1,		//Mximo nmero de vrtice.
      vertice;			//Vertice actual.
  float fvertice;		//Variable para guardar lo ledo desde el arhivo.

  rewind (GRAFODATA);		//Volver al inicio del archivo.

  //A continuacin, calculamos la cantidad de vrtices que contiene el grafo, de manera de crear, dinmicamente,
  //la matriz de adyacencias. Para hacer esto, buscamos el nmero de vrtice ms alto, aprovechndonos del hecho
  //que dichos nmeros son correlativos y comienzan en 0.
  while (fscanf(GRAFODATA, "%f", &fvertice) != EOF) //Para todos lo vrtices del archivo...
    {
      vertice = (int)fvertice;	//Aprovechando que los pesos estn en ]0;1[, hacemos un cast para eliminarlos
      if (vertice > maximo)	//...es mayor que el mayor nmero de vrtice hallado hasta el momento?
       {
         maximo = vertice;	//Entonces el ltimo vrtice ledo es el mximo.
       }//EndIf
    }//EndWhile
  return (maximo+1);		//Devolver la cantidad de vrtices.
}//EndCalc_N_Vertices


//Funcin Crear_Matriz 	: Crea una matriz de adyacencias.
//Variables de entrada 	: Descriptor del archivo con el grafo.
//Variables de salida  	: La matriz de adyacencia.
//Valor devuelto 	: La cantidad de vrtices en el grafo.
MatrizT Crear_Matriz (FILE *GRAFODATA, int N_Vertices)
{
  MatrizT Matriz;
  int vertice1,		//Primer vrtice
      vertice2,		//Segundo vrtice
      i,j;		//Contadores
  float peso;		//Peso del arco

  //Ahora creamos el espacio en memoria para la matriz.
  Matriz = malloc (N_Vertices*sizeof(float));		//Ahora creamos el espacio para la matriz...
  for (i = 0; i < N_Vertices; i++)
    {
      Matriz[i] = malloc (N_Vertices*sizeof(float));
      for (j = 0; j < N_Vertices; j++)
        {
          Matriz[i][j] = -1.0;				//...y la reseteamos con un valor que indique que...
        }//EndFor					//...no hay arco
    }//EndFor;

  //Ahora que tenemos el espacio en memoria listo, cargamos los datos a partir del archivo...
  rewind (GRAFODATA);					//Primero que nada, volvemos al principio...
  while (fscanf(GRAFODATA, "%d %d %f", &vertice1, &vertice2, &peso) != EOF)
     {
        Matriz[vertice1][vertice2] = peso;		//Guardamos el peso correspondiente a un arco en la matriz...
        Matriz[vertice2][vertice1] = peso;		//...y su posicin traspuesta
     }
  return (Matriz);					//Devolver la matriz de adyacencias.
}//EndCrear_Matriz


//Funcin Destruir_Matriz: Destruye una matriz de adyacencias y libera la memoria.
//Variables de entrada 	 : La matriz de adyacencia, cantidad de vrtices.
void Destruir_Matriz (MatrizT Matriz, int N_Vertices)
{
  int i;				//Contador

  for (i = 0; i < N_Vertices; i++)	//Liberar memoria.
    {
      free (Matriz[i]);
    }//EndFor
  free (Matriz);
}//EndDestruir_Matriz
